import 'dart:developer';

import 'package:flutter/material.dart';

class ClosureWidgetBuilder extends StatefulWidget {
  final ValueGetter<Widget> Function(
    BuildContext context,
    ValueSetter<VoidCallback> notify,
    ValueSetter<VoidCallback> capture,
  ) builder;

  const ClosureWidgetBuilder({
    Key key,
    @required this.builder,
  })  : assert(builder != null),
        super(key: key);

  @override
  _State createState() => _State();
}

class _State extends State<ClosureWidgetBuilder> {
  final List<VoidCallback> _disposes = [];
  ValueGetter<Widget> _widgetGetter;
  bool _hasDisposed = false;

  void _addDispose(VoidCallback callback) {
    assert(callback != null);
    _disposes.add(callback);
  }

  @override
  void dispose() {
    super.dispose();

    final disposes = List.from(_disposes);
    _disposes.clear();
    _hasDisposed = true;

    for (final fn in disposes) {
      try {
        fn();
      } catch (e) {
        log("Got error in disposing widget. fu: $fn, error: $e");
      }

      disposes.clear();
    }
  }

  void notify(VoidCallback callable) {
    if (!_hasDisposed) {
      setState(callable);
    } else {
      log("The widget has been disposed so the notify is ignored.");
    }
  }

  @override
  Widget build(BuildContext context) {
    if (_widgetGetter == null) {
      _widgetGetter = widget.builder(context, notify, this._addDispose);
      assert(_widgetGetter != null);
    }

    return _widgetGetter();
  }
}
